<font size="6">**交易系统各业务接口代码示例**</font><br>

<font size="6">文档说明</font>

<table><tbody>
    <tr>
        <th>日期</th><th>版本</th><th>修订摘要</th>
    </tr>
    <tr>
        <td>20241209</td><td>1.0.0</td><td>初版</td>
    </tr>
</table>




<font size="6">前言</font>

本接口规范用以指导开发通过 EMTAPI 对接的方式访问东财快速交易系统，进行现货/两融/期权各业务下单示例代码。

## 1. 普通股票委托示例


## 2. 新股申购委托示例


## 3. 配股配债委托示例


## 4. ETF申赎委托示例


## 5. 国债逆回购委托示例


## 6. 融资融券业务委托示例

### 6.1 <a>担保品买入</a>
``` cpp
uint64_t InsertOrder(TraderApi* m_pTraderApi, uint64_t session_id)
{
    //EMTOrderInsertInfo 为下单参数结构体
    EMTOrderInsertInfo order;
    //结构体初始化
    memset(&order, 0, sizeof(EMTOrderInsertInfo));
    //报单引用,由客户自定义
    order.order_client_id=111111;
    //股票代码(必须是担保证券,可调用API接口查询哪些是担保证券)
    std::string ticker = "300059";
    strcpy_s(order.ticker, EMT_TICKER_LEN, ticker.c_str());
    //市场参数
    order.market = EMT_MKT_SZ_A;
    //价格(可参考行情修改)
    order.price = 27.5;
    //委托数量
    order.quantity = 1000;
    //价格类型限价(可参考API文档定义类型)
    order.price_type = EMT_PRICE_LIMIT;
    //买卖方向(必须是EMT_SIDE_BUY)
    order.side = EMT_SIDE_BUY;
    //开平标志(除期权外,均使用EMT_POSITION_EFFECT_INIT)
    order.position_effect = EMT_POSITION_EFFECT_INIT;
    //业务类型(是EMT_BUSINESS_TYPE_MARGIN 融资融券业务或EMT_BUSINESS_TYPE_FIXPRICE 盘后定价业务)
    //如果是盘后定价业务,price_type价格类型不可为市价,且证券支持盘后定价
    order.business_type = EMT_BUSINESS_TYPE_MARGIN;
    //调用api接口委托前基本校验
    if(m_pTraderApi && session_id !=0 )
    {
        //返回报单在EMT系统中的ID,如果为“0”表示报单发送失败
        uint64_t nEmtId = m_pTraderApi->InsertOrder(&order, session_id);
        //下单发送失败
        if(nEmtId == 0)
        {
            //获取错误代码
            EMTRI * error = m_pTraderApi->GetApiLastError();
        }
    }
}
```

### 6.2 <a>担保品卖出</a>
``` cpp
uint64_t InsertOrder(TraderApi* m_pTraderApi, uint64_t session_id)
{
    //EMTOrderInsertInfo 为下单参数结构体
    EMTOrderInsertInfo order;
    //结构体初始化
    memset(&order, 0, sizeof(EMTOrderInsertInfo));
    //报单引用,由客户自定义
    order.order_client_id=111111;
    //股票代码
    std::string ticker = "300059";
    strcpy_s(order.ticker, EMT_TICKER_LEN, ticker.c_str());
    //市场参数
    order.market = EMT_MKT_SZ_A;
    //价格(可参考行情修改)
    order.price = 27.5;
    //委托数量
    order.quantity = 1000;
    //价格类型限价(可参考API文档定义类型)
    order.price_type = EMT_PRICE_LIMIT;
    //买卖方向(必须是EMT_SIDE_SELL)
    order.side = EMT_SIDE_SELL;
    //开平标志(除期权外,均使用EMT_POSITION_EFFECT_INIT)
    order.position_effect = EMT_POSITION_EFFECT_INIT;
    //业务类型(是EMT_BUSINESS_TYPE_MARGIN 融资融券业务或EMT_BUSINESS_TYPE_FIXPRICE 盘后定价业务)
    //如果是盘后定价业务,price_type价格类型不可为市价,且证券支持盘后定价
    order.business_type = EMT_BUSINESS_TYPE_MARGIN;
    //调用api接口委托前基本校验
    if(m_pTraderApi && session_id !=0 )
    {
        //返回报单在EMT系统中的ID,如果为“0”表示报单发送失败
        uint64_t nEmtId = m_pTraderApi->InsertOrder(&order, session_id);
        //下单发送失败
        if(nEmtId == 0)
        {
            //获取错误代码
            EMTRI * error = m_pTraderApi->GetApiLastError();
        }
    }
}
```

### 6.3 <a>融资买入</a>
``` cpp
uint64_t InsertOrder(TraderApi* m_pTraderApi, uint64_t session_id)
{
    //EMTOrderInsertInfo 为下单参数结构体
    EMTOrderInsertInfo order;
    //结构体初始化
    memset(&order, 0, sizeof(EMTOrderInsertInfo));
    //报单引用,由客户自定义
    order.order_client_id=111111;
    //股票代码(必须是融资融券标的,可调用API接口查询哪些是融资融券标的)
    std::string ticker = "300059";
    strcpy_s(order.ticker, EMT_TICKER_LEN, ticker.c_str());
    //市场参数
    order.market = EMT_MKT_SZ_A;
    //价格(可参考行情修改)
    order.price = 27.5;
    //委托数量
    order.quantity = 1000;
    //价格类型限价(可参考API文档定义类型)
    order.price_type = EMT_PRICE_LIMIT;
    //买卖方向(必须是EMT_SIDE_MARGIN_TRADE)
    order.side = EMT_SIDE_MARGIN_TRADE;
    //开平标志(除期权外,均使用EMT_POSITION_EFFECT_INIT)
    order.position_effect = EMT_POSITION_EFFECT_INIT;
    //业务类型(是EMT_BUSINESS_TYPE_MARGIN 融资融券业务或EMT_BUSINESS_TYPE_FIXPRICE 盘后定价业务)
    //如果是盘后定价业务,price_type价格类型不可为市价,且证券支持盘后定价
    order.business_type = EMT_BUSINESS_TYPE_MARGIN;
    //调用api接口委托前基本校验
    if(m_pTraderApi && session_id !=0 )
    {
        //返回报单在EMT系统中的ID,如果为“0”表示报单发送失败
        uint64_t nEmtId = m_pTraderApi->InsertOrder(&order, session_id);
        //下单发送失败
        if(nEmtId == 0)
        {
            //获取错误代码
            EMTRI * error = m_pTraderApi->GetApiLastError();
        }
    }
}
```

### 6.4 <a>融券卖出</a>
``` cpp
uint64_t InsertOrder(TraderApi* m_pTraderApi, uint64_t session_id)
{
    //EMTOrderInsertInfo 为下单参数结构体
    EMTOrderInsertInfo order;
    //结构体初始化
    memset(&order, 0, sizeof(EMTOrderInsertInfo));
    //报单引用,由客户自定义
    order.order_client_id=111111;
    //股票代码(必须是融资融券标的,可调用API接口查询哪些是融资融券标的)
    std::string ticker = "300059";
    strcpy_s(order.ticker, EMT_TICKER_LEN, ticker.c_str());
    //市场参数
    order.market = EMT_MKT_SZ_A;
    //价格(不得低于最新成交价)
    order.price = 27.5;
    //委托数量
    order.quantity = 1000;
    //价格类型限价(融券卖出必须是限价)
    order.price_type = EMT_PRICE_LIMIT;
    //买卖方向(必须是EMT_SIDE_SHORT_SELL)
    order.side = EMT_SIDE_SHORT_SELL;
    //开平标志(除期权外,均使用EMT_POSITION_EFFECT_INIT)
    order.position_effect = EMT_POSITION_EFFECT_INIT;
    //业务类型(是EMT_BUSINESS_TYPE_MARGIN 融资融券业务或EMT_BUSINESS_TYPE_FIXPRICE 盘后定价业务)
    //如果是盘后定价业务,price_type价格类型不可为市价,且证券支持盘后定价
    order.business_type = EMT_BUSINESS_TYPE_MARGIN;
    //调用api接口委托前基本校验
    if(m_pTraderApi && session_id !=0 )
    {
        //返回报单在EMT系统中的ID,如果为“0”表示报单发送失败
        uint64_t nEmtId = m_pTraderApi->InsertOrder(&order, session_id);
        //下单发送失败
        if(nEmtId == 0)
        {
            //获取错误代码
            EMTRI * error = m_pTraderApi->GetApiLastError();
        }
    }
}
```

### 6.5 <a>卖券还款</a>
``` cpp
uint64_t InsertOrder(TraderApi* m_pTraderApi, uint64_t session_id)
{
    //EMTOrderInsertInfo 为下单参数结构体
    EMTOrderInsertInfo order;
    //结构体初始化
    memset(&order, 0, sizeof(EMTOrderInsertInfo));
    //报单引用,由客户自定义
    order.order_client_id=111111;
    //股票代码
    std::string ticker = "300059";
    strcpy_s(order.ticker, EMT_TICKER_LEN, ticker.c_str());
    //市场参数
    order.market = EMT_MKT_SZ_A;
    //价格(可参考行情修改)
    order.price = 27.5;
    //委托数量
    order.quantity = 1000;
    //价格类型限价(可参考API文档定义类型)
    order.price_type = EMT_PRICE_LIMIT;
    //买卖方向(必须是EMT_SIDE_REPAY_MARGIN)
    order.side = EMT_SIDE_REPAY_MARGIN;
    //开平标志(除期权外,均使用EMT_POSITION_EFFECT_INIT)
    order.position_effect = EMT_POSITION_EFFECT_INIT;
    //业务类型(是EMT_BUSINESS_TYPE_MARGIN 融资融券业务或EMT_BUSINESS_TYPE_FIXPRICE 盘后定价业务)
    //如果是盘后定价业务,price_type价格类型不可为市价,且证券支持盘后定价
    order.business_type = EMT_BUSINESS_TYPE_MARGIN;
    //调用api接口委托前基本校验
    if(m_pTraderApi && session_id !=0 )
    {
        //返回报单在EMT系统中的ID,如果为“0”表示报单发送失败
        uint64_t nEmtId = m_pTraderApi->InsertOrder(&order, session_id);
        //下单发送失败
        if(nEmtId == 0)
        {
            //获取错误代码
            EMTRI * error = m_pTraderApi->GetApiLastError();
        }
    }
}
```

### 6.6 <a>买券还券</a>
``` cpp
uint64_t InsertOrder(TraderApi* m_pTraderApi, uint64_t session_id)
{
    //EMTOrderInsertInfo 为下单参数结构体
    EMTOrderInsertInfo order;
    //结构体初始化
    memset(&order, 0, sizeof(EMTOrderInsertInfo));
    //报单引用,由客户自定义
    order.order_client_id=111111;
    //股票代码
    std::string ticker = "300059";
    strcpy_s(order.ticker, EMT_TICKER_LEN, ticker.c_str());
    //市场参数
    order.market = EMT_MKT_SZ_A;
    //价格(可参考行情修改)
    order.price = 27.5;
    //委托数量(可能会产生余券)
    order.quantity = 1000;
    //价格类型限价(可参考API文档定义类型)
    order.price_type = EMT_PRICE_LIMIT;
    //买卖方向(必须是EMT_SIDE_REPAY_STOCK)
    order.side = EMT_SIDE_REPAY_STOCK;
    //开平标志(除期权外,均使用EMT_POSITION_EFFECT_INIT)
    order.position_effect = EMT_POSITION_EFFECT_INIT;
    //业务类型(是EMT_BUSINESS_TYPE_MARGIN 融资融券业务或EMT_BUSINESS_TYPE_FIXPRICE 盘后定价业务)
    //如果是盘后定价业务,price_type价格类型不可为市价,且证券支持盘后定价
    order.business_type = EMT_BUSINESS_TYPE_MARGIN;
    //调用api接口委托前基本校验
    if(m_pTraderApi && session_id !=0 )
    {
        //返回报单在EMT系统中的ID,如果为“0”表示报单发送失败
        uint64_t nEmtId = m_pTraderApi->InsertOrder(&order, session_id);
        //下单发送失败
        if(nEmtId == 0)
        {
            //获取错误代码
            EMTRI * error = m_pTraderApi->GetApiLastError();
        }
    }
}
```

### 6.7 <a>现券还券</a>
``` cpp
uint64_t InsertOrder(TraderApi* m_pTraderApi, uint64_t session_id)
{
    //EMTOrderInsertInfo 为下单参数结构体
    EMTOrderInsertInfo order;
    //结构体初始化
    memset(&order, 0, sizeof(EMTOrderInsertInfo));
    //报单引用,由客户自定义
    order.order_client_id=111111;
    //股票代码
    std::string ticker = "300059";
    strcpy_s(order.ticker, EMT_TICKER_LEN, ticker.c_str());
    //市场参数
    order.market = EMT_MKT_SZ_A;
    //委托数量(不可大于非当日的融券负债)
    order.quantity = 1000;
    //买卖方向(必须是EMT_SIDE_STOCK_REPAY_STOCK)
    order.side = EMT_SIDE_STOCK_REPAY_STOCK;
    //开平标志(除期权外,均使用EMT_POSITION_EFFECT_INIT)
    order.position_effect = EMT_POSITION_EFFECT_INIT;
    //业务类型(必须是EMT_BUSINESS_TYPE_MARGIN 融资融券业务)
    order.business_type = EMT_BUSINESS_TYPE_MARGIN;
    //调用api接口委托前基本校验
    if(m_pTraderApi && session_id !=0 )
    {
        //返回报单在EMT系统中的ID,如果为“0”表示报单发送失败
        uint64_t nEmtId = m_pTraderApi->InsertOrder(&order, session_id);
        //下单发送失败
        if(nEmtId == 0)
        {
            //获取错误代码
            EMTRI * error = m_pTraderApi->GetApiLastError();
        }
    }
}
```

### 6.8 <a>担保品转入</a>
``` cpp
uint64_t InsertOrder(TraderApi* m_pTraderApi, uint64_t session_id)
{
    //EMTOrderInsertInfo 为下单参数结构体
    EMTOrderInsertInfo order;
    //结构体初始化
    memset(&order, 0, sizeof(EMTOrderInsertInfo));
    //报单引用,由客户自定义
    order.order_client_id=111111;
    //股票代码(必须是担保证券,可调用API接口查询哪些是担保证券)
    std::string ticker = "300059";
    strcpy_s(order.ticker, EMT_TICKER_LEN, ticker.c_str());
    //市场参数
    order.market = EMT_MKT_SZ_A;
    //委托数量
    order.quantity = 1000;
    //买卖方向(必须是EMT_SIDE_GRTSTK_TRANSIN)
    order.side = EMT_SIDE_GRTSTK_TRANSIN;
    //开平标志(除期权外,均使用EMT_POSITION_EFFECT_INIT)
    order.position_effect = EMT_POSITION_EFFECT_INIT;
    //业务类型(必须是EMT_BUSINESS_TYPE_MARGIN 融资融券业务)
    order.business_type = EMT_BUSINESS_TYPE_MARGIN;
    //调用api接口委托前基本校验
    if(m_pTraderApi && session_id !=0 )
    {
        //返回报单在EMT系统中的ID,如果为“0”表示报单发送失败
        uint64_t nEmtId = m_pTraderApi->InsertOrder(&order, session_id);
        //下单发送失败
        if(nEmtId == 0)
        {
            //获取错误代码
            EMTRI * error = m_pTraderApi->GetApiLastError();
        }
    }
}
```

### 6.9 <a>担保品转出</a>
``` cpp
uint64_t InsertOrder(TraderApi* m_pTraderApi, uint64_t session_id)
{
    //EMTOrderInsertInfo 为下单参数结构体
    EMTOrderInsertInfo order;
    //结构体初始化
    memset(&order, 0, sizeof(EMTOrderInsertInfo));
    //报单引用,由客户自定义
    order.order_client_id=111111;
    //股票代码
    std::string ticker = "300059";
    strcpy_s(order.ticker, EMT_TICKER_LEN, ticker.c_str());
    //市场参数
    order.market = EMT_MKT_SZ_A;
    //委托数量(信用资金购买的证券无法转出,且转出的维保比不能低于提取线)
    order.quantity = 1000;
    //买卖方向(必须是EMT_SIDE_GRTSTK_TRANSOUT)
    order.side = EMT_SIDE_GRTSTK_TRANSOUT;
    //开平标志(除期权外,均使用EMT_POSITION_EFFECT_INIT)
    order.position_effect = EMT_POSITION_EFFECT_INIT;
    //业务类型(必须是EMT_BUSINESS_TYPE_MARGIN 融资融券业务)
    order.business_type = EMT_BUSINESS_TYPE_MARGIN;
    //调用api接口委托前基本校验
    if(m_pTraderApi && session_id !=0 )
    {
        //返回报单在EMT系统中的ID,如果为“0”表示报单发送失败
        uint64_t nEmtId = m_pTraderApi->InsertOrder(&order, session_id);
        //下单发送失败
        if(nEmtId == 0)
        {
            //获取错误代码
            EMTRI * error = m_pTraderApi->GetApiLastError();
        }
    }
}
```

### 6.10 <a>直接还款</a>
``` cpp
uint64_t CreditCashRepay(TraderApi* m_pTraderApi, uint64_t session_id)
{
    //调用api接口委托前基本校验
    if(m_pTraderApi && session_id !=0 )
    {
        double amount =10000;
        //返回报单在EMT系统中的ID,如果为“0”表示报单发送失败
        uint64_t nEmtId = m_pTraderApi->CreditCashRepay(amount, session_id);
        //下单发送失败
        if(nEmtId == 0)
        {
            //获取错误代码
            EMTRI * error = m_pTraderApi->GetApiLastError();
        }
    }
}
```

### 6.11 <a>指定负债还款</a>
``` cpp
uint64_t CreditCashRepaySpecifiedDebt(TraderApi* m_pTraderApi, uint64_t session_id)
{
    //EMTCrdRepaySpecifiedDebtReq 为指定负债还款结构体
    EMTCrdRepaySpecifiedDebtReq order;
    //结构体初始化
    memset(&order, 0, sizeof(EMTCrdRepaySpecifiedDebtReq));
    //偿还金额
    order.request_amount=10000;    
    //合约开仓日期
    order.order_date = 20241205;
    //合约编号
    std::string ticker = "BA00000001";
    strcpy_s(order.debt_id, 16, ticker.c_str());

    //调用api接口委托前基本校验
    if(m_pTraderApi && session_id !=0 )
    {
        //返回报单在EMT系统中的ID,如果为“0”表示报单发送失败
        uint64_t nEmtId = m_pTraderApi->CreditCashRepaySpecifiedDebt(&order, session_id);
        //下单发送失败
        if(nEmtId == 0)
        {
            //获取错误代码
            EMTRI * error = m_pTraderApi->GetApiLastError();
        }
    }
}
```

### 6.12 <a>资金划拨</a>
``` cpp
uint64_t FundTranser(TraderApi* m_pTraderApi, uint64_t session_id)
{
    //EMTFundTransferReq 为指资金划拨请求结构体
    EMTFundTransferReq order;
    //结构体初始化
    memset(&order, 0, sizeof(EMTFundTransferReq));
    //资金账号
    std::string fund_account = "100000001";
    strcpy_s(order.fund_account, EMT_ACCOUNT_NAME_LEN, fund_account.c_str());
    //划拨金额
    order.amount = 1000;
    //划拨类型(支持类型参考API文档EMT_FUND_TRANSFER_TYPE)
    order.transfer_type = EMT_FUND_TRANSFER_OUT;
    //调用api接口委托前基本校验
    if(m_pTraderApi && session_id !=0 )
    {
        //返回报单在EMT系统中的ID,如果为“0”表示报单发送失败
        uint64_t nEmtId = m_pTraderApi->FundTranser(&order, session_id);
        //下单发送失败
        if(nEmtId == 0)
        {
            //获取错误代码
            EMTRI * error = m_pTraderApi->GetApiLastError();
        }
    }
}
```

### 6.13 <a>额度调拨</a>
``` cpp
//该接口只适用于两地分仓的额度调拨
uint64_t CreditQuotaTransfer(TraderApi* m_pTraderApi, uint64_t session_id)
{
    //EMTQuotaTransferReq 为两地分仓信用额度调拨请求
    EMTQuotaTransferReq order;
    //结构体初始化
    memset(&order, 0, sizeof(EMTQuotaTransferReq));
    //资金账号
    std::string fund_account = "100000001";
    strcpy_s(order.fund_account, EMT_ACCOUNT_NAME_LEN, fund_account.c_str());
    //额度划拨金额
    order.amount = 1000;
    //额度划拨类型(支持类型参考API文档EMT_QUOTA_TRANSFER_TYPE)
    order.transfer_type = EMT_STK_QUOTA_TRANSFER_OUT;

    //调用api接口委托前基本校验
    if(m_pTraderApi && session_id !=0 )
    {
        //返回报单在EMT系统中的ID,如果为“0”表示报单发送失败
        uint64_t nEmtId = m_pTraderApi->CreditQuotaTransfer(&order, session_id);
        //下单发送失败
        if(nEmtId == 0)
        {
            //获取错误代码
            EMTRI * error = m_pTraderApi->GetApiLastError();
        }
    }
}
```

# 7. 期权业务委托示例